/* colmain.skel Skeleton for collection main program
 * $Log: colmain.skel,v $
 * Revision 1.20  1995/10/18  02:05:30  nort
 * Changes to nlcon_display()
 *
 * Revision 1.19  1995/10/06  16:40:02  nort
 * Changes to nlcon_disp()
 * Old change to ROWS() and FRACSECS() macros
 *
 * Revision 1.18  1994/08/29  20:29:15  nort
 * Added check for obsolete MSG_LABEL
 *
 * Revision 1.17  1994/08/29  20:15:12  nort
 * Mods for OUI: Moved several initializations into oui_init_options()
 * Made OPT_COL_INIT Col_init_options() and CONSOLE_INIT obsolete
 * Changed calls to msg() to nl_error(), making TMC collection independent
 * of the msg module, since nl_error is made to be redefined.
 *
 * Revision 1.16  1994/02/15  05:56:10  nort
 * Added some verbosity to COL_get_pointer()
 * I don't know why seteuid(getuid()) has been commented out:
 * I should try it again later.
 *
 * Revision 1.15  1993/09/27  18:53:20  nort
 * Added seteuid(getuid())
 * Cleaned up RCS log
 * Added cmrcsid[]
 * Added include <string.h> for memcpy prototype
 *
 * Revision 1.14  1993/03/08  18:11:45  nort
 * Added timerbd timing when available
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/kernel.h>
#include <sys/timers.h>
#include <sys/proxy.h>
#include <time.h>
#include <unistd.h>
#include "nortlib.h"
#include "oui.h"
#include "dbr.h"
#include "msg.h"
#include "globmsg.h"
#include "cc.h"
#include "collect.h"
#include "timerbd.h"
#pragma off (unreferenced)
  static char cmrcsid[] =
	"$Id: colmain.skel,v 1.20 1995/10/18 02:05:30 nort Exp nort $";
#pragma on (unreferenced)



 typedef unsigned short UINT;


 typedef unsigned char UCHAR;
	unsigned short InjectingInc = 0;


 typedef unsigned long ULONG;
ULONG CHROMtime;
#include "convfunc.h"
#include "ttdriver.h"
#include "defcals.h"

int tt_p4=72;
int tt_p3=48;
int tt_p2=24;
int tt_p1=0;




	
	

	 typedef float adT;
	 typedef float adT_flow;
	 typedef float vdc_chan;
	 typedef float vdc_pow;
	 typedef float adT_pump;

	 typedef float dacMSpr;

	 typedef UINT InjStateT;

float read_atod(int atod_chan) {
	
	unsigned short atod_val;
	float voltage = -99999;

	atod_val = tt_read_atod( atod_chan );
	atod_val = atod_val + 2048;
	atod_val = atod_val & 0xFFF;

	if (atod_val >= 2048)	{
		atod_val = atod_val|(0xF000);
		atod_val=~atod_val;
		++atod_val;
		voltage = -10.0 * ((float) atod_val) / 2048.0;
		//	printf("reading atod channel %i value is negative %i \n",atod_chan, atod_val);
	} else 	{
		voltage = 10.0 * ((float) atod_val) / 2048.0;
	//	printf("reading atod channel %i value is %f \n",atod_chan,voltage);

	}	

	return voltage;
}


float read_temp(int address) { 
	float voltage;

	voltage = read_atod(address);

	return convert_temps(voltage);
}


float read_temp12v(int address) { 
	float voltage;

	voltage = read_atod(address);

	return convert_temps12v(voltage);
}

float read_ad_value(int address, float R1, float V1, float V0) { 
	float Vx;

	Vx = read_atod(address);

	if (Vx < 0)
		Vx = -Vx;

	return convert_VtoR( Vx, R1, V1, V0 );

}



#include "valves.h"
#include "address.h"





	 typedef unsigned char DigOutWd;
	 typedef DigOutWd DigInWd;



	 typedef short DigBit;
	 typedef DigBit Bit_HI_LO;
	 typedef DigBit Bit_Op_Cl;
	 typedef DigBit Bit_Cl_Op;
	 typedef DigBit Bit_On_Off;
	 typedef DigBit Bit_Off_On;
	 typedef DigBit Bit_Air_Cal;
	 typedef DigBit Bit_offline;


	UCHAR MSDstateS;

int read_dig_in (unsigned short dig_chan)  {
	
	unsigned short dig_val;
	unsigned short address;
	int value;

	address=dig_chan/8;	
	dig_val=tt_read_digital(address);
	address=dig_chan-address*8;
	value=1<<address;
	//	address=pow(2,address);
	dig_val=dig_val&value;
	if (dig_val==0)
		return 0;
	return 1;
}

int read_dig_out(int chan) {

	return read_dig_in(chan+64);
}


int fail_board(int digch) {
	static int fail_toggle = 0;
	
	if (fail_toggle==0) {
		tt_scdc_command(100 + digch);
		fail_toggle = 1;
		return fail_toggle;
	}

	fail_toggle = 0;
	tt_scdc_command(digch);
	return fail_toggle;
}

// used to pulse a digch (hard coded for dig 20 and 21
// and MSDstate 4 and 5).
int trapToggle( int digch ) {
	static int pulse1 = 0;
	static int pulse2 = 0;
	int	pulse, state;
	const int trap1add = 20;
	const int trap2add = 21;

	if (digch == trap1add) {
	 	state = 4;
		pulse = pulse1;
	} else if (digch == trap2add) {
		state = 5;
		pulse = pulse2;
	} else {
		return 0;
	}

	// toggle dig line
	if ( MSDstateS == state ) {
		switch ( pulse ) {
			case 0:		// off, turn on
				tt_scdc_command( 100 + digch );
				pulse = 1;
				break;
			case 1:		// on, turn off
				tt_scdc_command( digch );
				pulse = 0;
				break;
		}
	} else {
		if ( pulse == 1 ) {
			tt_scdc_command( digch );
			pulse = 0;
		}
	}

	if (digch == trap1add)
		pulse1 = pulse;
	else
		pulse2 = pulse;

	return read_dig_out( digch );

}


#include <sys/types.h>
pid_t rs485_proxy;
short Omega485[34];

short OmegaArray( int add ) {
	return Omega485[add];
}



 typedef short OmegaT;

 typedef short OmegaTsp;
#include <math.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/types.h>
#include <stdlib.h>
#include "/home/panther/drivers/smart/smart.h"
#include "regnames.h"
#include "panthermsg.h"

#define 		ECDOFFLINE 		-9999
#define			NUMSECSXAXIS	 150
#define			ELECRATE		0.1			// seconds

pid_t			ECDresp_proxy;

chrom_packet 	ECDresp[MAXDET];

long int 		DetAvgCt1=0, DetAvgCt2=0, DetAvgCt3=0, DetAvgCt4=0,
		 		DetAvgOffline1=0, DetAvgOffline2=0, DetAvgOffline3=0, DetAvgOffline4=0,
	 			ecd_sum1=0, ecd_sum2=0, ecd_sum3=0, ecd_sum4=0,
				ecdA_SF6save=0, ecdA_F11save=0, ecdA_COsave=0, ecdA_PANsave=0;
float			ecdSF6x = 0.0, ecdF11x = 0.0, ecdCOx = 0.0, ecdPANx = 0.0;
int				currPoint[MAXDET];

// function gets the data from shared data array with smart driver
long int GC_Ctr_Pull( int elecADD ) {
	long ecdDATA;
	
	if ( currPoint[elecADD] >= MAXPOINTS ) currPoint[elecADD] = 0;

	// new data
	if ( currPoint[elecADD] < ECDresp[elecADD].numpnts ) {
		ecdDATA = ECDresp[elecADD].resp[currPoint[elecADD]];
		currPoint[elecADD]++;
	} 
	else {
		ecdDATA = ECDOFFLINE;
		currPoint[elecADD] = 0;
		ECDresp[elecADD].numpnts = -1;
	}

	return ecdDATA;	
}




 typedef long GC_Freq;
 typedef long GC_Ctr;

GC_Freq ecdA_SF6, ecdA_F11, ecdA_CO, ecdA_PAN;

 typedef unsigned char UNYBL;


 typedef unsigned long InjFile;

InjFile ITXFileNum;




ULONG CUMSEC;
#include <sys/types.h>
pid_t valco_proxy;
int valvePos[10] = {0,0,3,3,3,3,3,3,3,3};


 typedef short valvT;
 typedef DigBit valvetext;
 typedef DigBit valveMS1text;
 typedef DigBit valveMS2text;
#include <math.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "da_cache.h"
#include "regnames.h"

#define MAXMSDCHANS 	  8		// Max number of MS channels
#define MAXMSDIONS 		 30		// Max number of ions per channel
#define MAXMSDPOINTS 	 30		// Max Number of data points in a second for an ion
#define MSDOFFLINE 	  -9999
#define MSDMISSING	   -999
#define MAXSENDMSG     2000		// max number of chars from the msd server
#define SEPCHARS		":,()"
#define MSDIONFILE		"/home/panther/flight/msd.ions"

pid_t		MSDresp_proxy;

long int 	MSD_Data_Array[MAXMSDCHANS][MAXMSDPOINTS],
			MSD_Time_Array[MAXMSDPOINTS],
			msdTIMEoldPlot,
			MSchanAvg[MAXMSDCHANS],
			msd_sum[MAXMSDCHANS],
			MSD_ChanX;

unsigned int 	currMSDPoint[MAXMSDCHANS],
				chanInc;

unsigned char 	MSDresp[MAXSENDMSG];		// data returned by msd

struct MSwindow {
	float	ion;			// ion number
	float	dwell;			// dwell time (0 < dwell <= 1) (fraction of a second)
} MStable[MAXMSDCHANS][MAXMSDIONS];


int ReadMSDionFile ( char *IONFILE ) {
	FILE 	*fp;
	char 	buffer[1000];
	char 	*p;
	char	tmp[1];
	int		chan, inc;
	static char	once;
	double	ion, dwell;

	if (once == 1) return 0;

	fp = fopen(	IONFILE, "r" );
	if (fp == NULL) {
		fprintf(stderr, "Open failed: Cannont find %s\n", IONFILE);
		return -1;
	} else {
		while ( fgets( buffer, 1000, fp ) != NULL ) {

			// First strip out the MS channel number
			p = strstr( strlwr(buffer), "mschan" );
			strncpy(tmp, p + strlen("MSchan"), 1);
			chan = atoi(tmp);
			if (chan >= MAXMSDCHANS)
				break;
		
			// Next strip out the ions and dwell times
			p =	strtok(buffer, SEPCHARS );
			inc = 0;
			while ( p != NULL ) {

				if ( strstr(strlwr(p), "null") != NULL ) {
					MStable[chan][inc].ion = 0;
					MStable[chan][inc].dwell = 0;
					inc++;
				} else {
					ion = atof(p);
					if (ion >= 1) { 		// found ion
						p = strtok( NULL, SEPCHARS );
						dwell = atof(p);
						if (( dwell <= 1 ) && ( dwell > 0 )) {
							MStable[chan][inc].ion = ion;
							MStable[chan][inc].dwell = dwell;
							//printf ("%d,%d: %f %f\n", chan, inc, ion, dwell);
						} else {
							MStable[chan][inc].ion = 0;
							MStable[chan][inc].dwell = 0;
							fprintf(stderr, "Bad dwell time for channel %d, ion %f\n", chan, ion);
						}

						inc++;
					} else {
						MStable[chan][inc].ion = 0;
						MStable[chan][inc].dwell = 0;
						//printf("null ion: chan = %d, inc = %d\n", chan,inc);
					}

				}

				p = strtok( NULL, SEPCHARS );
			}
		}
	}

	fclose(fp);
	once = 1;

	return 0;

}

// Function takes a ion number and returns the MS channel number for
// a given time window.
int ion2chan( int win, float ion ) {

	int ch;
	float tol = 0.001;

	// Out of range variable
	if (win > MAXMSDIONS)
		return -1;

	// Scan MStable array for match
	for(ch=0; ch < MAXMSDCHANS; ch++) {
		if (((MStable[ch][win].ion - tol) < ion) && ((MStable[ch][win].ion + tol) > ion))
			return ch;
	}

	// No match in this time window
	return -1;

}

// Functions to pull data from MSD_Data and Time arrays.
long int MSD_Chan_Pull( int chan, int type ) {
	long msdDATA;
	static long msdTIMEold;

	if (currMSDPoint[chan] > MAXMSDPOINTS) 
		currMSDPoint[chan] = MAXMSDPOINTS;

	// type 0 is msd resp data, type 1 is msd time data.
	if ( type == 0 ) {
		msdDATA = MSD_Data_Array[chan][currMSDPoint[chan]];
		currMSDPoint[chan]++;
	} 
	else if ( type == 1 ) {		// real time displayed on text string.
		msdDATA = MSD_Time_Array[currMSDPoint[chan]-1];
		if ( msdDATA == MSDMISSING )
			msdDATA = msdTIMEold;
		else
			msdTIMEold = msdDATA;
	}
	else if ( type == 2 ) {		// this time needs to be monitonically increasing for rtg.
		msdDATA = MSD_Time_Array[currMSDPoint[chan]-1];
		if ( msdDATA == MSDMISSING )
			msdDATA = msdTIMEoldPlot++;
		else if ( msdDATA <= msdTIMEoldPlot ) {
			if ( msdDATA > msdTIMEoldPlot )
				msdDATA = msdDATA + 10;	
			else
				msdDATA = msdTIMEoldPlot + 10;
		}
		msdTIMEoldPlot = msdDATA;
	}

	return msdDATA;	
}




 typedef unsigned short WinNum;

 typedef long MSD_DataType;

MSD_DataType msdA_0, msdA_1, msdA_2, msdA_3, msdA_4, msdA_5, msdA_6, msdA_7, msdA_8;

int MSD_load( unsigned short IonWinNum ) {

	char *p;
	unsigned int chaninc[MAXMSDCHANS], inc, i, c;
	int chan = 0;
	float ion;

	// Initialize global currMSDPoint to 0
	for(c=0; c < MAXMSDCHANS; c++) {
		chaninc[c] = 0;
		currMSDPoint[c] = 0;
	}

	// Fills in MSD_Data_Array with "missing" data MSDMISSING
	for(i=0; i < MAXMSDPOINTS; i++) {
		for(c=0; c < MAXMSDCHANS; c++) {
			MSD_Data_Array[c][i] = MSDMISSING;
			if (c == 0)
				MSD_Time_Array[i] = MSDMISSING;
		}
	}

	// Parse data returned in string MSDresp
	p = strtok(MSDresp, ",");
	inc = 0;
	while( p != NULL ) {

		// Get ion number
		if ( (inc % 3) == 0 ) {
			ion = atof(p);
			chan = ion2chan(IonWinNum, ion);
			// printf("ion = %f, chan = %d\n", ion, chan);
		} 

		// response data
		else if ( (inc % 3) == 1 ) {
			// Set data array
			if ((chan >= 0) && (chan < MAXMSDCHANS)) {
				MSD_Data_Array[chan][chaninc[chan]] = atol(p);
				chaninc[chan]++;
				if (chaninc[chan] > MAXMSDPOINTS) {
					chaninc[chan] = MAXMSDPOINTS;
					break;
				}
			}
		} 

		// time data
		else if ( (inc % 3) == 2 ) {
			if (chan == 0)
				MSD_Time_Array[chaninc[chan] - 1] = atol(p);
		} 

		p = strtok(NULL, ",");
		inc++;
	}

	// Set MSDresp array to 0
	for (i=0; i<MAXSENDMSG; i++) 
		MSDresp[i] = '\0';
	
	return 1;

}		



	unsigned short InjectingInc_MSD = 0;


ULONG MSDchromtime;
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include "regnames.h"

#define MAXMON 	19		// should match number in msd driver

pid_t	MSDeng_proxy;
float 	MSDeng[MAXMON];

void InitializeMSDeng() {
	int param;
	static char once;

	if (once == 1) return;

	for (param=0; param<MAXMON; param++)
		MSDeng[param] = -99.0;

	once = 1;

}




 typedef double MSDdouble;
#include <sys/types.h>
#include "maycomm.h"
pid_t maycomm_proxy = 0;

TDLstruct tdl;

#define USELONGPATH  375	// use the long path TDL when less than 900 ppm
#define OffsetShortPath  150   // Offset to match long and short at switch
int P;



 typedef float tdlfloat;
 typedef float tdlfloas;
 typedef short tdlint6;
  #include "ttdriver.h"
  #define DG_CUSTOM_INIT tt_gc_reset()
  #define N_INJS_PER_FILE 1




UINT CurrInj_, CurrFile_, N_Inj_in_File_, Injected_;
UINT CurrInjMSD_, CurrFileMSD_, N_Inj_in_FileMSD_, InjectedMSD_;
  #include <ctype.h>
  static UINT read_gcnum( FILE *fp ) {
    int c;
	UINT v = 0;
	
    c = getc(fp);
	while ( isspace(c) ) c = getc(fp);
	while ( isdigit(c) ) {
	  v = v*10 + c - '0';
	  c = getc(fp);
	}
	return v;
  }

  static UINT read_msdnum( FILE *fp ) {
    int c;
	UINT v = 0;
	
    c = getc(fp);
	while ( isspace(c) ) c = getc(fp);
	while ( isdigit(c) ) {
	  v = v*10 + c - '0';
	  c = getc(fp);
	}
	return v;
  }

  void read_gcfile( UINT *injno, UINT *fileno ) {
	FILE *fp;
	
	fp = fopen( "GCFILE.CFG", "r" );
	if ( fp != 0 ) {
	  *injno = read_gcnum( fp );
	  *fileno = read_gcnum( fp );
	  fclose( fp );
	}
  }

  void read_msdfile( UINT *injno, UINT *fileno ) {
	FILE *fp;
	
	fp = fopen( "MSDFILE.CFG", "r" );
	if ( fp != 0 ) {
	  *injno = read_msdnum( fp );
	  *fileno = read_msdnum( fp );
	  fclose( fp );
	}
  }

  void write_gcfile( UINT injno, UINT fileno ) {
	FILE *fp;
	
	fp = fopen( "GCFILE.CFG", "w" );
	fprintf( fp, "%u\n%u\n\n\n", injno, fileno );
	fclose( fp );
  }

  void write_msdfile( UINT injno, UINT fileno ) {
	FILE *fp;
	
	fp = fopen( "MSDFILE.CFG", "w" );
	fprintf( fp, "%u\n%u\n\n\n", injno, fileno );
	fclose( fp );
  }


  static char nullregion[1];
  static unsigned char far *GCstatePtr = &nullregion;
  static unsigned char far *MSDstatePtr = &nullregion;

#ifdef OPT_COL_INIT
  #error OPT_COL_INIT is obsolete
#endif
#ifdef MSG_LABEL
  #error MSG_LABEL is obsolete
#endif

#ifdef N_CONSOLES
  #include "nl_cons.h"
  #ifndef DATA_ATTR
	#define DATA_ATTR 7
  #endif

  #define cdisplay(d,r,c,s) nlcon_display(d,r,c,s,DATA_ATTR)
  #define display_(d,r,c,t,v,s) {\
	static t sv_;\
	if (v != sv_) { cdisplay(d,r,c,s); sv_ = v; }\
  }
#endif

#ifdef CONSOLE_INIT
  #error CONSOLE_INIT is obsolete
#endif


unsigned char V0 = 0x1;
unsigned char V1 = 0x1;
unsigned char V2 = 0x1;
unsigned char V3 = 0x1;
unsigned char V4 = 0x1;
unsigned char V5 = 0x1;
unsigned char V6 = 0x1;
unsigned char V7 = 0x1;
unsigned char V8 = 0x1;
UINT CurrInj;
UINT CurrFile;
UCHAR GCstate;
UCHAR MSDstate;
UINT MFCtr, Synch;
InjStateT InjState;
adT_pump rpm_Ipump;
adT PresHead_P;
adT_flow FlowM_CO;
adT_flow FlowB_CO;
adT PresBP_CO;
adT PresE_CO;
vdc_chan VoltM_CO;
vdc_chan VoltB_CO;
vdc_chan VoltBP_CO;
vdc_chan VoltE_CO;
adT_flow FlowM_SF6;
adT_flow FlowB_SF6;
adT PresBP_SF6;
adT PresE_SF6;
vdc_chan VoltM_SF6;
vdc_chan VoltB_SF6;
vdc_chan VoltBP_SF6;
vdc_chan VoltE_SF6;
adT_flow FlowM_F11;
adT_flow FlowB_F11;
adT PresBP_F11;
adT PresE_F11;
vdc_chan VoltM_F11;
vdc_chan VoltB_F11;
vdc_chan VoltBP_F11;
vdc_chan VoltE_F11;
adT Pres1_PAN;
adT Pres2_PAN;
adT PresSrc_PAN;
adT_flow FlowSrc_PAN;
adT PresBP_PAN;
adT PresE_PAN;
vdc_chan VoltM_PAN;
vdc_chan VoltB_PAN;
vdc_chan VoltBP_PAN;
vdc_chan VoltE_PAN;
adT PresMS1;
adT PresMS2;
adT PresBP_5;
adT PresE_5;
vdc_chan VoltMS1;
vdc_chan VoltMS2;
vdc_chan VoltBP_5;
vdc_chan VoltE_5;
dacMSpr PresMS1sp;
dacMSpr PresMS2sp;
adT Trap1_set;
adT Trap1_re;
adT Trap2_set;
adT Trap2_re;
adT ColT_MS1;
adT ColT_MS2;
adT ColT_PAN;
adT Pres_SL;
adT Pres_Inlet;
adT Flow_MSsl;
adT Flow_Packsl;
adT Flow_PANsl;
adT PresH_CO2;
adT PresH_N2O;
adT PresH_ZA;
adT PresH_PAN;
adT PresH_Cal2;
adT PresH_Cal1;
adT PresH_He;
adT PresH_N2;
adT PresL_CO2;
adT PresL_N2O;
adT PresL_ZA;
adT PresL_PAN;
adT PresL_Cal2;
adT PresL_Cal1;
adT PresL_He;
adT PresL_N2;
adT Temp_F11sl;
adT Temp_SF6sl;
adT Temp_COsl;
adT Temp_PANsl;
adT Temp_GSVbo;
adT Temp_elect;
adT Temp_stack;
adT Temp_Ipump;
adT Temp_PANsrc;
adT Temp_GasBot;
adT Temp_PANlmp;
vdc_pow Pow_p28pri;
vdc_pow Pow_p28sec;
vdc_pow Pow_p15pel1;
vdc_pow Pow_p15pel2;
vdc_pow Pow_p12;
vdc_pow Pow_m15;
vdc_pow Pow_p15;
vdc_pow Pow_p5;
vdc_pow Pow_p24gsv;
vdc_pow Pow_p24ms;
Bit_HI_LO faildummy;
Bit_Cl_Op SLoopIN;
Bit_Cl_Op SLoopOUT;
Bit_Cl_Op PANflsh;
Bit_Op_Cl SLoopMSout;
Bit_HI_LO pilotTrip;
Bit_Off_On pelt1State;
Bit_Off_On pelt2State;
Bit_Off_On PANlamp;
Bit_Off_On PANflowCont;
Bit_Off_On trap1heat;
Bit_Off_On trap2heat;
Bit_Off_On trap1relay;
Bit_Off_On trap2relay;
Bit_Air_Cal PANsol1;
Bit_Cl_Op PANsol2;
Bit_Off_On PANsol3;
Bit_Op_Cl PANsol4;
OmegaT F11_Col;
OmegaT CO_Col;
OmegaT SF6_Col;
OmegaT SF6_Post;
OmegaT F11_ECD;
OmegaT CO_ECD;
OmegaT SF6_ECD;
OmegaT Trap1_CB;
OmegaT Trap2_CB;
OmegaT PAN_ECD;
OmegaTsp F11_Col_Sp;
OmegaTsp CO_Col_Sp;
OmegaTsp SF6_Col_Sp;
OmegaTsp SF6_Post_Sp;
OmegaTsp F11_ECD_Sp;
OmegaTsp CO_ECD_Sp;
OmegaTsp SF6_ECD_Sp;
OmegaTsp Trap1_CB_Sp;
OmegaTsp Trap2_CB_Sp;
OmegaTsp PAN_ECD_Sp;
GC_Ctr GC_Ctr_SF6;
GC_Ctr GC_Ctr_F11;
GC_Ctr GC_Ctr_CO;
GC_Ctr GC_Ctr_PAN;
valvT ssv1;
valvT ssv2;
valvetext valv_F11;
valvetext valv_CO;
valvetext valv_PAN;
valvetext valv_SF6;
valveMS1text valv_MS1;
valveMS2text valv_MS2;
WinNum IonWinNum;
UINT loadMSDarray;
MSD_DataType MSD_Chan0;
MSD_DataType MSD_Chan1;
MSD_DataType MSD_Chan2;
MSD_DataType MSD_Chan3;
MSD_DataType MSD_Chan4;
MSD_DataType MSD_Chan5;
MSD_DataType MSD_Time;
MSD_DataType MSD_TimePlot;
UINT CurrInj_MSD;
UINT CurrFile_MSD;
MSDdouble msdECUR;
MSDdouble msdIPOL;
MSDdouble msdREP;
MSDdouble msdEEGY;
MSDdouble msdAMUG;
MSDdouble msdAMUO;
MSDdouble msdIFOC;
MSDdouble msdENT;
MSDdouble msdFSEL;
MSDdouble msdMMW;
MSDdouble msdENTO;
MSDdouble msdHEDEN;
MSDdouble msdCAL;
MSDdouble msdEMUL;
MSDdouble msdSRCTEMP;
MSDdouble msdQUAD;
MSDdouble msdTURBOPCT;
MSDdouble msdMASG;
MSDdouble msdMASO;
ULONG maycX;
tdlfloas tdlP;
tdlfloas tdlT;
tdlfloat tdlsH2O;
tdlint6  tdlsAmp;
tdlint6  tdlsPow;
tdlint6  tdlsPos;
tdlint6  tdlsZer;
tdlfloat tdllH2O;
tdlint6  tdllAmp;
tdlint6  tdllPow;
tdlint6  tdllPos;
tdlint6  tdllZer;
tdlfloat tdlbH2O;
union {
  struct {
    UINT MFCtr;
    GC_Ctr GC_Ctr_SF6;
    vdc_chan VoltE_CO;
    InjStateT InjState;
    Bit_Op_Cl SLoopMSout;
    vdc_chan VoltB_CO;
    vdc_chan VoltM_CO;
    adT PresE_CO;
    adT PresHead_P;
    UINT CurrFile;
    Bit_Off_On pelt1State;
    UINT CurrInj;
    Bit_HI_LO pilotTrip;
    adT_flow FlowM_SF6;
    adT PresMS1;
    UINT Synch;
  } U0;
  struct {
    char U2[2];
    MSD_DataType MSD_TimePlot;
    vdc_chan VoltE_F11;
    vdc_chan VoltBP_CO;
    vdc_chan VoltBP_SF6;
    vdc_chan VoltB_SF6;
    vdc_chan VoltM_SF6;
    adT PresBP_CO;
    adT_pump rpm_Ipump;
    adT_flow FlowM_CO;
    adT PresMS2;
    adT Trap1_re;
  } U1;
  struct {
    char U4[6];
    vdc_chan VoltE_PAN;
    vdc_chan VoltE_SF6;
    vdc_chan VoltB_F11;
    vdc_chan VoltM_F11;
    adT PresE_F11;
    adT PresE_SF6;
    adT_flow FlowB_CO;
    adT_flow FlowB_SF6;
    adT ColT_MS1;
    adT ColT_MS2;
  } U3;
  struct {
    char U6[6];
    adT Trap1_set;
    vdc_chan VoltBP_F11;
    vdc_chan VoltM_PAN;
    adT_flow FlowSrc_PAN;
    adT PresBP_PAN;
    adT PresBP_F11;
    adT PresBP_SF6;
    adT_flow FlowM_F11;
    adT ColT_PAN;
    adT PresH_N2;
  } U5;
  struct {
    char U8[6];
    adT PresL_N2;
    vdc_chan VoltB_PAN;
    vdc_chan VoltE_5;
    vdc_chan VoltBP_PAN;
    adT PresE_PAN;
    adT PresSrc_PAN;
    adT_flow FlowB_F11;
    adT Pres1_PAN;
    adT PresH_He;
    adT PresL_He;
  } U7;
  struct {
    char U10[6];
    adT Temp_F11sl;
    adT Trap2_re;
    adT Trap2_set;
    vdc_chan VoltBP_5;
    vdc_chan VoltMS2;
    vdc_chan VoltMS1;
    adT Pres2_PAN;
    adT PresBP_5;
    adT PresL_CO2;
    adT Temp_elect;
  } U9;
  struct {
    char U12[6];
    vdc_pow Pow_p28sec;
    adT PresL_N2O;
    adT Pres_Inlet;
    adT Pres_SL;
    dacMSpr PresMS1sp;
    dacMSpr PresMS2sp;
    adT PresE_5;
    adT PresH_CO2;
    adT Temp_COsl;
    vdc_pow Pow_p28pri;
  } U11;
  struct {
    char U14[6];
    Bit_Off_On trap1heat;
    Bit_Off_On trap2heat;
    adT Temp_GasBot;
    adT PresH_N2O;
    adT PresH_PAN;
    adT Flow_PANsl;
    adT Flow_Packsl;
    adT Flow_MSsl;
    adT PresL_Cal2;
    adT Temp_stack;
    vdc_pow Pow_p15pel2;
  } U13;
  struct {
    char U16[6];
    OmegaT SF6_Post;
    OmegaTsp SF6_Post_Sp;
    vdc_pow Pow_p5;
    adT PresL_PAN;
    adT PresL_ZA;
    adT PresH_ZA;
    adT PresH_Cal1;
    adT PresH_Cal2;
    adT Temp_SF6sl;
    vdc_pow Pow_p15pel1;
    vdc_pow Pow_p12;
  } U15;
  struct {
    char U18[6];
    valveMS2text valv_MS2;
    valveMS1text valv_MS1;
    Bit_HI_LO faildummy;
    valvetext valv_CO;
    adT Temp_GSVbo;
    adT Temp_Ipump;
    adT Temp_PANlmp;
    adT Temp_PANsl;
    adT PresL_Cal1;
    vdc_pow Pow_p24gsv;
    vdc_pow Pow_m15;
    vdc_pow Pow_p15;
  } U17;
  struct {
    char U20[6];
    MSD_DataType MSD_Time;
    Bit_Cl_Op SLoopIN;
    WinNum IonWinNum;
    Bit_Cl_Op SLoopOUT;
    OmegaT Trap1_CB;
    Bit_Off_On PANflowCont;
    Bit_Cl_Op PANflsh;
    Bit_Off_On PANlamp;
    Bit_Air_Cal PANsol1;
    Bit_Off_On trap2relay;
    UCHAR GCstate;
    UCHAR MSDstate;
    adT Temp_PANsrc;
    Bit_Off_On pelt2State;
    UINT CurrInj_MSD;
    vdc_pow Pow_p24ms;
    OmegaT F11_ECD;
    OmegaTsp F11_ECD_Sp;
  } U19;
  struct {
    char U22[6];
    MSDdouble msdEEGY;
    OmegaT PAN_ECD;
    OmegaTsp PAN_ECD_Sp;
    OmegaTsp Trap1_CB_Sp;
    Bit_Off_On trap1relay;
    OmegaT CO_Col;
    OmegaTsp CO_Col_Sp;
    Bit_Cl_Op PANsol2;
    Bit_Off_On PANsol3;
    Bit_Op_Cl PANsol4;
    UINT CurrFile_MSD;
    GC_Ctr GC_Ctr_CO;
    OmegaT F11_Col;
    OmegaTsp F11_Col_Sp;
    OmegaT SF6_ECD;
    OmegaTsp SF6_ECD_Sp;
  } U21;
  struct {
    char U24[6];
    MSDdouble msdFSEL;
    valvetext valv_F11;
    valvetext valv_SF6;
    valvT ssv2;
    valvT ssv1;
    OmegaT Trap2_CB;
    OmegaTsp Trap2_CB_Sp;
    OmegaT CO_ECD;
    OmegaTsp CO_ECD_Sp;
    MSD_DataType MSD_Chan0;
    tdlfloas tdlP;
    OmegaT SF6_Col;
    OmegaTsp SF6_Col_Sp;
    GC_Ctr GC_Ctr_PAN;
  } U23;
  struct {
    char U26[6];
    MSDdouble msdEMUL;
    MSD_DataType MSD_Chan4;
    MSD_DataType MSD_Chan3;
    MSD_DataType MSD_Chan2;
    MSD_DataType MSD_Chan1;
    MSDdouble msdECUR;
    GC_Ctr GC_Ctr_F11;
    ULONG maycX;
  } U25;
  struct {
    char U28[6];
    MSDdouble msdMASO;
    MSDdouble msdREP;
    MSDdouble msdIPOL;
    MSDdouble msdAMUO;
    MSDdouble msdAMUG;
  } U27;
  struct {
    char U30[6];
    tdlint6  tdllZer;
    tdlint6  tdllPos;
    valvetext valv_PAN;
    UINT loadMSDarray;
    MSDdouble msdENT;
    MSDdouble msdIFOC;
    MSDdouble msdENTO;
    MSDdouble msdMMW;
  } U29;
  struct {
    char U32[10];
    MSD_DataType MSD_Chan5;
    MSDdouble msdCAL;
    MSDdouble msdHEDEN;
    MSDdouble msdQUAD;
    MSDdouble msdSRCTEMP;
  } U31;
  struct {
    char U34[10];
    tdlint6  tdllPow;
    tdlint6  tdllAmp;
    MSDdouble msdMASG;
    MSDdouble msdTURBOPCT;
    tdlfloas tdlT;
    tdlfloat tdlbH2O;
  } U33;
  struct {
    char U36[14];
    tdlint6  tdlsZer;
    tdlint6  tdlsPos;
    tdlint6  tdlsPow;
    tdlint6  tdlsAmp;
    tdlfloat tdllH2O;
    tdlfloat tdlsH2O;
  } U35;
} *home;

static void VSET_3_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 2: /* MaycommTog */
      V8 |= 0x1;
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 2: /* MaycommTog */
      V8 &= ~0x1;
      break;
  }
}
static void VSET_2_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 3: /* Injection_MSD */
      V6 |= 0x1;
      break;
    case 2: /* Injecting_MSD */ {
        V7 |= 0x1;
        V5 |= 0x1;
      }
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 3: /* Injection_MSD */
      V6 &= ~0x1;
      break;
    case 2: /* Injecting_MSD */ {
        V7 &= ~0x1;
        V5 &= ~0x1;
        {
        	msdTIMEoldPlot = 0;
        } {
          if ( CurrFileMSD_ == 0 )
        	read_msdfile( &CurrInjMSD_, &CurrFileMSD_ );
          CurrInjMSD_++;
          if ( N_Inj_in_FileMSD_ == N_INJS_PER_FILE-1 ) {
            N_Inj_in_FileMSD_ = 0;
        	CurrFileMSD_++;
          } else N_Inj_in_FileMSD_++;
          write_msdfile( CurrInjMSD_, CurrFileMSD_ );
        }
      }
      break;
    case 1: /* NotInjecting_MSD */ {
        {
        	ReadMSDionFile( MSDIONFILE );
        } {
        	InitializeMSDeng();
        }
      }
      break;
  }
}
static void VSET_1_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 2: /* GCTog */
      V3 |= 0x1;
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 2: /* GCTog */
      V3 &= ~0x1;
      break;
  }
}
static void VSET_0_(unsigned short newstate) {
  static unsigned short oldstate;
  
  switch (oldstate) {
    case 3: /* Injection */
      V1 |= 0x1;
      break;
    case 2: /* Injecting */ {
        V4 |= 0x1;
        V2 |= 0x1;
        V0 |= 0x1;
      }
      break;
  }
  oldstate = newstate;
  switch (newstate) {
    case 3: /* Injection */
      V1 &= ~0x1;
      break;
    case 2: /* Injecting */ {
        V4 &= ~0x1;
        V2 &= ~0x1;
        V0 &= ~0x1;
        {
          if ( CurrFile_ == 0 )
        	read_gcfile( &CurrInj_, &CurrFile_ );
          CurrInj_++;
          if ( N_Inj_in_File_ == N_INJS_PER_FILE-1 ) {
            N_Inj_in_File_ = 0;
        	CurrFile_++;
          } else N_Inj_in_File_++;
          write_gcfile( CurrInj_, CurrFile_ );
        }
      }
      break;
  }
}
static void tminitfunc(void) {VSET_0_(1);VSET_1_(1);VSET_2_(1);VSET_3_(1);
}
#define TMINITFUNC tminitfunc()

static void nullfunc(void);
static void CF1_0(void) {
  home->U0.MFCtr = MFCtr;
  home->U0.Synch = Synch;
}

static void BF2_0(void) {
  CF1_0();
  {
  	MSD_Chan0 = MSD_Chan_Pull(0, 0);
  	MSD_Chan1 = MSD_Chan_Pull(1, 0);
  	MSD_Chan2 = MSD_Chan_Pull(2, 0);
  	MSD_Chan3 = MSD_Chan_Pull(3, 0);
  	MSD_Chan4 = MSD_Chan_Pull(4, 0);
  	MSD_Chan5 = MSD_Chan_Pull(5, 0);
  	MSD_Time  = MSD_Chan_Pull(0, 1);
  	MSD_TimePlot = MSD_Chan_Pull(0, 2);
  }
  GC_Ctr_PAN = GC_Ctr_Pull(0x4);
  GC_Ctr_CO = GC_Ctr_Pull(0x3);
  GC_Ctr_F11 = GC_Ctr_Pull(0x2);
  home->U23.MSD_Chan0 = MSD_Chan0;
  home->U25.MSD_Chan1 = MSD_Chan1;
  home->U25.MSD_Chan2 = MSD_Chan2;
  home->U25.MSD_Chan3 = MSD_Chan3;
  home->U25.MSD_Chan4 = MSD_Chan4;
  home->U31.MSD_Chan5 = MSD_Chan5;
  home->U19.MSD_Time = MSD_Time;
  home->U1.MSD_TimePlot = MSD_TimePlot;
  home->U23.GC_Ctr_PAN = GC_Ctr_PAN;
  home->U25.GC_Ctr_F11 = GC_Ctr_F11;
  home->U21.GC_Ctr_CO = GC_Ctr_CO;
  {
  
  	for(chanInc=0; chanInc<MAXMSDCHANS; chanInc++) {
  		MSD_ChanX = MSDMISSING;
  		switch( chanInc ) {
  			case 0:
  				MSD_ChanX = MSD_Chan0; break;
  			case 1:
  				MSD_ChanX = MSD_Chan1; break;
  			case 2:
  				MSD_ChanX = MSD_Chan2; break;
  			case 3:
  				MSD_ChanX = MSD_Chan3; break;
  			case 4:
  				MSD_ChanX = MSD_Chan4; break;
  			case 5:
  				MSD_ChanX = MSD_Chan5; break;
  			case 6:
  				MSD_ChanX = MSD_Chan5; break;
  			case 7:
  				MSD_ChanX = MSD_Chan5; break;
  			default:
  				chanInc = MAXMSDCHANS-1; MSD_ChanX = MSDMISSING; break;
  		}
  
  		if ( MSD_ChanX > 0 ) {
  			msd_sum[chanInc] += MSD_ChanX;
  			MSchanAvg[chanInc]++;
  		}
  	}
  }
}

static void BF32_0(void) {
  BF2_0();
  {if (!(V1)) {
    { CHROMtime += 1; InjectingInc = 0; }
  }if (!(V2)) {
    { 
    	if (InjectingInc == 0 ) {
    		CHROMtime = 1; 
    		InjectingInc = 1;
    	}
    	else CHROMtime += 1;
      }
  }
  }
  {	
  	if (rs485_proxy != 0)  Trigger ( rs485_proxy );
  }
  {
  	if (ECDresp_proxy != 0) Trigger ( ECDresp_proxy );
  }
  {if (!(V3)) {
    { VSET_1_(1);
    	}
  }if (!(V4)) {
    V4 |= 0x1;
    { VSET_1_(2);
    		ecdSF6x = 0.0;
    		ecdF11x = 0.0;
    		ecdCOx  = 0.0;
    		ecdPANx = 0.0;
    	}
  }
  
  	if ( ecdSF6x > NUMSECSXAXIS ) { VSET_1_(2);
  		ecdSF6x = 0.0;
  		ecdF11x = 0.0;
  		ecdCOx  = 0.0;
  		ecdPANx = 0.0;
  	}
  }
  {
  	if ( DetAvgCt1 > 0 ) {
  		ecdA_SF6 = ecd_sum1/DetAvgCt1;
  		ecdA_SF6save = ecdA_SF6;
  		DetAvgCt1 = 0;
  		ecd_sum1 = 0;
  		DetAvgOffline1 = 0;
  	} 
  	else if (( ecdA_SF6save > 0 ) && ( DetAvgOffline1 < 1 )) {
  		DetAvgOffline1++;
  		ecdA_SF6 = ecdA_SF6save;
  	}
  	else {
  		ecdA_SF6 = ECDOFFLINE;
  		ecdA_SF6save = 0;
  	}
  
  	if ( DetAvgCt2 > 0 ) {
  		ecdA_F11 = ecd_sum2/DetAvgCt2;
  		ecdA_F11save = ecdA_F11;
  		DetAvgCt2 = 0;
  		ecd_sum2 = 0;
  		DetAvgOffline2 = 0;
  	} 
  	else if (( ecdA_F11save > 0 ) && ( DetAvgOffline2 < 1 )) {
  		DetAvgOffline2++;
  		ecdA_F11 = ecdA_F11save;
  	}
  	else {
  		ecdA_F11 = ECDOFFLINE;
  		ecdA_F11save = 0;
  	}
  
  	if ( DetAvgCt3 > 0 ) {
  		ecdA_CO = ecd_sum3/DetAvgCt3;
  		ecdA_COsave = ecdA_CO;
  		DetAvgCt3 = 0;
  		ecd_sum3 = 0;
  		DetAvgOffline3 = 0;
  	} 
  	else if (( ecdA_COsave > 0 ) && ( DetAvgOffline3 < 1 )) {
  		DetAvgOffline3++;
  		ecdA_CO = ecdA_COsave;
  	}
  	else {
  		ecdA_CO = ECDOFFLINE;
  		ecdA_COsave = 0;
  	}
  
  	if ( DetAvgCt4 > 0 ) {
  		ecdA_PAN = ecd_sum4/DetAvgCt4;
  		ecdA_PANsave = ecdA_PAN;
  		DetAvgCt4 = 0;
  		ecd_sum4 = 0;
  		DetAvgOffline4 = 0;
  	} 
  	else if (( ecdA_PANsave > 0 ) && ( DetAvgOffline4 < 1 )) {
  		DetAvgOffline4++;
  		ecdA_PAN = ecdA_PANsave;
  	}
  	else {
  		ecdA_PAN = ECDOFFLINE;
  		ecdA_PANsave = 0;
  	}
  
  }
  {
   	++CUMSEC;
  }
  {
  	if ( MSchanAvg[0] > 0 ) {
  		msdA_0 = msd_sum[0]/MSchanAvg[0];
  		MSchanAvg[0] = msd_sum[0] = 0;
  	} else {
  		msdA_0 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[1] > 0 ) {
  		msdA_1 = msd_sum[1]/MSchanAvg[1];
  		MSchanAvg[1] = msd_sum[1] = 0;
  	} else {
  		msdA_1 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[2] > 0 ) {
  		msdA_2 = msd_sum[2]/MSchanAvg[2];
  		MSchanAvg[2] = msd_sum[2] = 0;
  	} else {
  		msdA_2 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[3] > 0 ) {
  		msdA_3 = msd_sum[3]/MSchanAvg[3];
  		MSchanAvg[3] = msd_sum[3] = 0;
  	} else {
  		msdA_3 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[4] > 0 ) {
  		msdA_4 = msd_sum[4]/MSchanAvg[4];
  		MSchanAvg[4] = msd_sum[4] = 0;
  	} else {
  		msdA_4 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[5] > 0 ) {
  		msdA_5 = msd_sum[5]/MSchanAvg[5];
  		MSchanAvg[5] = msd_sum[5] = 0;
  	} else {
  		msdA_5 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[6] > 0 ) {
  		msdA_6 = msd_sum[6]/MSchanAvg[6];
  		MSchanAvg[6] = msd_sum[6] = 0;
  	} else {
  		msdA_6 = MSDOFFLINE;
  	}
  
  	if ( MSchanAvg[7] > 0 ) {
  		msdA_7 = msd_sum[7]/MSchanAvg[7];
  		MSchanAvg[7] = msd_sum[7] = 0;
  	} else {
  		msdA_7 = MSDOFFLINE;
  	}
  
  }
  {if (!(V6)) {
    { 
    	MSDchromtime += 1; 
    	InjectingInc_MSD = 0;
      }
  }if (!(V7)) {
    { 
    	if ( InjectingInc_MSD == 0 ) {
    		MSDchromtime = 1; 
    		InjectingInc_MSD = 1;
    	} else 
    		MSDchromtime += 1;
      }
  }
  }
  {
  	if (maycomm_proxy != 0)  Trigger ( maycomm_proxy );
  }
}

static void CF2_1(void) {
  CF1_0();
  GC_Ctr_SF6 = GC_Ctr_Pull(0x1);
  home->U0.GC_Ctr_SF6 = GC_Ctr_SF6;
}

static void EF2_1(void) {
  {
  	if (GC_Ctr_SF6 >= 0) {
  		ecd_sum1 += GC_Ctr_SF6;
  		DetAvgCt1++;
  		ecdSF6x += ELECRATE;
  	} 
  	if (GC_Ctr_F11 >= 0) {
  		ecd_sum2 += GC_Ctr_F11;
  		DetAvgCt2++;
  		ecdF11x += ELECRATE;
  	}
  	if (GC_Ctr_CO >= 0) {
  		ecd_sum3 += GC_Ctr_CO;
  		DetAvgCt3++;
  		ecdCOx += ELECRATE;
  	}
  	if (GC_Ctr_PAN >= 0) {
  		ecd_sum4 += GC_Ctr_PAN;
  		DetAvgCt4++;
  		ecdPANx += ELECRATE;
  	}
  }
}

static void CF8_1(void) {
  CF2_1();
  trap2heat = trapToggle(21);
  trap1heat = trapToggle(20);
  home->U13.trap2heat = trap2heat;
  home->U13.trap1heat = trap1heat;
}

static void CF16_1(void) {
  CF8_1();
  {
  	loadMSDarray = MSD_load( IonWinNum );
  	if ( MSDresp_proxy != 0 )
  		Trigger( MSDresp_proxy );
  }
  faildummy = fail_board(13);
  home->U29.loadMSDarray = loadMSDarray;
  home->U17.faildummy = faildummy;
}

static void BF32_1(void) {
  CF16_1();
  {
  		PresMS1  = read_ad_value( tt_p3 + 0, presL30_R1, presL30_V1, presL30_V0 );
  		PresMS2  = read_ad_value( tt_p3 + 2, presL30_R1, presL30_V1, presL30_V0 );
      	PresBP_5 = read_ad_value( tt_p3 + 4, presH_R1, presH_V1, presH_V0 );
  		PresE_5  = read_ad_value( tt_p3 + 16, presL_R1, presL_V1, presL_V0 );
  		VoltMS1  = read_atod( tt_p3 + 1 );
  		VoltMS2  = read_atod( tt_p3 + 3 );
      	VoltBP_5 = read_atod( tt_p3 + 5 );
  	    VoltE_5  = read_atod( tt_p3 + 17 );
  	}
  home->U0.PresMS1 = PresMS1;
  home->U1.PresMS2 = PresMS2;
  home->U9.PresBP_5 = PresBP_5;
  home->U11.PresE_5 = PresE_5;
  home->U9.VoltMS1 = VoltMS1;
  home->U9.VoltMS2 = VoltMS2;
  home->U9.VoltBP_5 = VoltBP_5;
  home->U7.VoltE_5 = VoltE_5;
  EF2_1();
}

static void BF32_3(void) {
  CF2_1();
  ColT_MS2 = read_ad_value( tt_p1 + 1, tempTrap_R1, tempTrap_V1, tempTrap_V0 );
  ColT_MS1 = read_ad_value( tt_p1 + 0, tempTrap_R1, tempTrap_V1, tempTrap_V0 );
  {
  		FlowM_CO  = read_ad_value( tt_p4 + 0, flowH_R1, flowH_V1, flowH_V0 );
  		FlowB_CO  = read_ad_value( tt_p4 + 2, flowH_R1, flowH_V1, flowH_V0 );
  	    PresBP_CO = read_ad_value( tt_p4 + 4, presH_R1, presH_V1, presH_V0 );
  		
  		PresE_CO  = read_ad_value( tt_p3 + 12, presL_R1, presL_V1, presL_V0 );
  		VoltM_CO  = read_atod( tt_p4 + 1 );
  		VoltB_CO  = read_atod( tt_p4 + 3 );
  	    VoltBP_CO = read_atod( tt_p4 + 5 );
      	VoltE_CO  = read_atod( tt_p3 + 13 );
  	}
  home->U1.FlowM_CO = FlowM_CO;
  home->U3.FlowB_CO = FlowB_CO;
  home->U1.PresBP_CO = PresBP_CO;
  home->U0.PresE_CO = PresE_CO;
  home->U0.VoltM_CO = VoltM_CO;
  home->U0.VoltB_CO = VoltB_CO;
  home->U1.VoltBP_CO = VoltBP_CO;
  home->U0.VoltE_CO = VoltE_CO;
  home->U3.ColT_MS2 = ColT_MS2;
  home->U3.ColT_MS1 = ColT_MS1;
  EF2_1();
}

static void BF32_5(void) {
  CF2_1();
  ColT_PAN = read_ad_value( tt_p1 + 3, tempTrap_R1, tempTrap_V1, tempTrap_V0 );
  {
  		FlowM_F11  = read_ad_value( tt_p4 + 12, flowH_R1, flowH_V1, flowH_V0 );
  		FlowB_F11  = read_ad_value( tt_p4 + 14, flowH_R1, flowH_V1, flowH_V0 );
  	    PresBP_F11 = read_ad_value( tt_p4 + 16, presH_R1, presH_V1, presH_V0 );
  		
  		PresE_F11  = read_ad_value( tt_p3 + 12, presL_R1, presL_V1, presL_V0 );
  		VoltM_F11  = read_atod( tt_p4 + 13 );
  		VoltB_F11  = read_atod( tt_p4 + 15 );
  	    VoltBP_F11 = read_atod( tt_p4 + 17 );
      	VoltE_F11  = read_atod( tt_p3 + 13 );
  	}
  home->U5.FlowM_F11 = FlowM_F11;
  home->U7.FlowB_F11 = FlowB_F11;
  home->U5.PresBP_F11 = PresBP_F11;
  home->U3.PresE_F11 = PresE_F11;
  home->U3.VoltM_F11 = VoltM_F11;
  home->U3.VoltB_F11 = VoltB_F11;
  home->U5.VoltBP_F11 = VoltBP_F11;
  home->U1.VoltE_F11 = VoltE_F11;
  home->U25.maycX = maycX;
  home->U5.ColT_PAN = ColT_PAN;
  EF2_1();
  maycX++;
  {if (!(V8)) {
    { VSET_3_(1); }
  }
  
  	if ( maycX >= 120 ) { 
  		maycX = 0; VSET_3_(2);
  	}
  }
}

static void BF32_7(void) {
  CF2_1();
  Pow_m15 = read_atod( tt_p2 + 21 ) * 10;
  Pow_p12 = read_atod( tt_p2 + 18 ) * 10;
  {
  		Pres1_PAN  = read_ad_value( tt_p4 + 18, presL30_R1, presL30_V1, presL30_V0 );
  		Pres2_PAN  = read_ad_value( tt_p4 + 20, presL30_R1, presL30_V1, presL30_V0 );
  		PresSrc_PAN = read_ad_value( tt_p2 + 16, presPne30_R1, presPne30_V1, presPne30_V0 );
  		PresE_PAN  = read_ad_value( tt_p3 + 14, presL_R1, presL_V1, presL_V0 );
  		FlowSrc_PAN = read_ad_value( tt_p2 + 12, flowPne_R1, flowPne_V1, flowPne_V0 );
  		VoltM_PAN  = read_atod( tt_p4 + 19 );
  		VoltB_PAN  = read_atod( tt_p4 + 21 );
  	    VoltE_PAN  = read_atod( tt_p3 + 15 );
      	PresBP_PAN = read_ad_value( tt_p4 + 22, presL30_R1, presL30_V1, presL30_V0 );
      	VoltBP_PAN = read_atod( tt_p4 + 23 );
  	}
  home->U7.Pres1_PAN = Pres1_PAN;
  home->U9.Pres2_PAN = Pres2_PAN;
  home->U7.PresSrc_PAN = PresSrc_PAN;
  home->U7.PresE_PAN = PresE_PAN;
  home->U5.FlowSrc_PAN = FlowSrc_PAN;
  home->U5.VoltM_PAN = VoltM_PAN;
  home->U7.VoltB_PAN = VoltB_PAN;
  home->U3.VoltE_PAN = VoltE_PAN;
  home->U15.Pow_p12 = Pow_p12;
  home->U17.Pow_m15 = Pow_m15;
  EF2_1();
}

static void BF32_9(void) {
  CF8_1();
  Pow_p15 = read_atod( tt_p2 + 22 ) * 10;
  {
  		FlowM_SF6  = read_ad_value( tt_p4 +  6, flowH_R1, flowH_V1, flowH_V0 );
  		FlowB_SF6  = read_ad_value( tt_p4 +  8, flowH_R1, flowH_V1, flowH_V0 );
      	PresBP_SF6 = read_ad_value( tt_p4 + 10, presH_R1, presH_V1, presH_V0 );
  		PresE_SF6  = read_ad_value( tt_p3 + 10, presL_R1, presL_V1, presL_V0 );
  		VoltM_SF6  = read_atod( tt_p4 + 7 );
  		VoltB_SF6  = read_atod( tt_p4 + 9 );
      	VoltBP_SF6 = read_atod( tt_p4 + 11 );
      	VoltE_SF6  = read_atod( tt_p3 + 11 );
  	}
  home->U0.FlowM_SF6 = FlowM_SF6;
  home->U3.FlowB_SF6 = FlowB_SF6;
  home->U5.PresBP_SF6 = PresBP_SF6;
  home->U3.PresE_SF6 = PresE_SF6;
  home->U1.VoltM_SF6 = VoltM_SF6;
  home->U1.VoltB_SF6 = VoltB_SF6;
  home->U1.VoltBP_SF6 = VoltBP_SF6;
  home->U3.VoltE_SF6 = VoltE_SF6;
  home->U17.Pow_p15 = Pow_p15;
  EF2_1();
}

static void BF32_11(void) {
  CF2_1();
  {
  	tdlP = tdl.p;
  	tdlT = tdl.t;
  	tdlsH2O = tdl.s.h2o;
  	tdllH2O = tdl.l.h2o;
  	tdlsAmp = tdl.s.amp;
  	tdlsPow = tdl.s.pow;
  	tdlsPos = tdl.s.pos;
  	tdlsZer = tdl.s.zero;
  	tdllAmp = tdl.l.amp;
  	tdllPow = tdl.l.pow;
  	tdllPos = tdl.l.pos;
  	tdllZer = tdl.l.zero;
  }
  Pow_p15pel2 = read_atod( tt_p2 + 15 ) * 10;
  Pow_p15pel1 = read_atod( tt_p2 + 14 ) * 10;
  home->U23.tdlP = tdlP;
  home->U33.tdlT = tdlT;
  home->U35.tdlsH2O = tdlsH2O;
  home->U35.tdllH2O = tdllH2O;
  home->U35.tdlsAmp = tdlsAmp;
  home->U35.tdlsPow = tdlsPow;
  home->U35.tdlsPos = tdlsPos;
  home->U35.tdlsZer = tdlsZer;
  home->U33.tdllAmp = tdllAmp;
  home->U33.tdllPow = tdllPow;
  home->U29.tdllPos = tdllPos;
  home->U29.tdllZer = tdllZer;
  home->U13.Pow_p15pel2 = Pow_p15pel2;
  home->U15.Pow_p15pel1 = Pow_p15pel1;
  EF2_1();
}

static void BF32_13(void) {
  CF2_1();
  {
  
  	ssv1 = valvePos[0];
  	ssv2 = valvePos[1];
  	valv_F11 = valvePos[2]-1;
  	valv_CO = valvePos[3]-1;
  	valv_PAN = valvePos[4]-1;
  	valv_SF6 = valvePos[5]-1;
  	valv_MS1 = valvePos[6]-1;
  	valv_MS2 = valvePos[7]-1;
  
  	if (valco_proxy != 0) 
  		Trigger ( valco_proxy );    
  
  }
  Pow_p24ms = read_atod( tt_p2 + 20 ) * 10;
  Pow_p24gsv = read_atod( tt_p2 + 19 ) * 10;
  Pow_p28pri = read_atod( tt_p2 + 17 ) * 10;
  {
  		Flow_MSsl   = read_ad_value( tt_p1 + 6, flowPne_R1, flowPne_V1, flowPne_V0 );
  		Flow_Packsl = read_ad_value( tt_p1 + 5, flowH_R1, flowH_V1, flowH_V0 );
  		Flow_PANsl  = read_ad_value( tt_p1 + 4, flowH_R1, flowH_V1, flowH_V0 );
  	}
  home->U13.Flow_MSsl = Flow_MSsl;
  home->U13.Flow_Packsl = Flow_Packsl;
  home->U13.Flow_PANsl = Flow_PANsl;
  home->U23.ssv1 = ssv1;
  home->U23.ssv2 = ssv2;
  home->U23.valv_SF6 = valv_SF6;
  home->U23.valv_F11 = valv_F11;
  home->U17.valv_CO = valv_CO;
  home->U29.valv_PAN = valv_PAN;
  home->U17.valv_MS1 = valv_MS1;
  home->U17.valv_MS2 = valv_MS2;
  home->U11.Pow_p28pri = Pow_p28pri;
  home->U19.Pow_p24ms = Pow_p24ms;
  home->U17.Pow_p24gsv = Pow_p24gsv;
  EF2_1();
}

static void BF32_15(void) {
  CF2_1();
  Pow_p5 = read_atod( tt_p2 + 23 );
  Pow_p28sec = read_atod( tt_p2 + 13 ) * 10;
  PresH_N2   = read_ad_value( tt_p1 + 23, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  PresH_He   = read_ad_value( tt_p1 + 21, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  PresH_Cal1 = read_ad_value( tt_p1 + 19, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  PresH_Cal2 = read_ad_value( tt_p1 + 17, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  PresH_CO2  = read_ad_value( tt_p1 +  9, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  Pres_Inlet = read_ad_value(tt_p3 + 6, presL_R1, presL_V1, presL_V0 );
  Pres_SL    = read_ad_value(tt_p3 + 8, presL_R1, presL_V1, presL_V0 );
  home->U5.PresH_N2 = PresH_N2;
  home->U7.PresH_He = PresH_He;
  home->U11.PresH_CO2 = PresH_CO2;
  home->U15.PresH_Cal2 = PresH_Cal2;
  home->U15.PresH_Cal1 = PresH_Cal1;
  home->U5.PresBP_PAN = PresBP_PAN;
  home->U11.Pres_SL = Pres_SL;
  home->U11.Pres_Inlet = Pres_Inlet;
  home->U15.Pow_p5 = Pow_p5;
  home->U11.Pow_p28sec = Pow_p28sec;
  EF2_1();
}

static void BF32_17(void) {
  CF16_1();
  PresL_He   = read_ad_value( tt_p1 + 20, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresL_Cal1 = read_ad_value( tt_p1 + 18, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresL_Cal2 = read_ad_value( tt_p1 + 16, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresL_CO2  = read_ad_value( tt_p1 +  8, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresH_PAN  = read_ad_value( tt_p1 + 15, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  PresH_ZA   = read_ad_value( tt_p1 + 13, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  PresH_N2O  = read_ad_value( tt_p1 + 11, presCylH_R1, presCylH_V1, presCylH_V0 ) - 15;
  PresHead_P = read_ad_value(tt_p1 + 2, presL_R1, presL_V1, presL_V0 );
  home->U7.PresL_He = PresL_He;
  home->U9.PresL_CO2 = PresL_CO2;
  home->U13.PresL_Cal2 = PresL_Cal2;
  home->U17.PresL_Cal1 = PresL_Cal1;
  home->U0.PresHead_P = PresHead_P;
  home->U15.PresH_ZA = PresH_ZA;
  home->U13.PresH_PAN = PresH_PAN;
  home->U13.PresH_N2O = PresH_N2O;
  EF2_1();
}

static void BF32_19(void) {
  CF2_1();
  Temp_elect = read_temp( tt_p2 + 4 );
  Temp_COsl  = read_temp( tt_p2 + 2 );
  PresL_N2   = read_ad_value( tt_p1 + 22, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresL_PAN  = read_ad_value( tt_p1 + 14, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresL_ZA   = read_ad_value( tt_p1 + 12, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresL_N2O  = read_ad_value( tt_p1 + 10, presCylL_R1, presCylL_V1, presCylL_V0 ) - 15;
  PresMS2sp = convert_VtoR( 10.0 * tt_dac_in(0xD) / 4095.0, presL30_R1, presL30_V1, presL30_V0 );
  PresMS1sp = convert_VtoR( 10.0 * tt_dac_in(0xC) / 4095.0, presL30_R1, presL30_V1, presL30_V0 );
  rpm_Ipump = read_ad_value(tt_p1+7, 7000, 11, 0);
  home->U9.Temp_elect = Temp_elect;
  home->U11.Temp_COsl = Temp_COsl;
  home->U33.tdlbH2O = tdlbH2O;
  home->U1.rpm_Ipump = rpm_Ipump;
  home->U11.PresMS2sp = PresMS2sp;
  home->U11.PresMS1sp = PresMS1sp;
  home->U15.PresL_ZA = PresL_ZA;
  home->U15.PresL_PAN = PresL_PAN;
  home->U11.PresL_N2O = PresL_N2O;
  home->U7.PresL_N2 = PresL_N2;
  EF2_1();
  {
  //	P = ((PresE_SF6) - 30);
      P = (tdlP);
  	tdlbH2O = tdllH2O / ( 1.15 - 0.0008123 * P + 5.283e-7 * P*P );
  //	tdlbH2O = tdllH2O / ( 1.135 - 0.000757 * P + 5.372e-7 * P*P );
  	tdlbH2O = (tdllH2O * 1.62) -2.55;
  	
  	if ( tdllH2O > USELONGPATH ) tdlbH2O = (tdlsH2O - OffsetShortPath)*1.55;
  }
}

static void BF32_21(void) {
  CF2_1();
  Temp_PANlmp = read_temp12v( tt_p2 + 10 );
  Temp_GasBot = read_temp12v( tt_p2 + 9 );
  Temp_PANsrc = read_temp12v( tt_p2 + 8 );
  Temp_Ipump = read_temp( tt_p2 + 7 );
  Temp_stack = read_temp( tt_p2 + 6 );
  Temp_GSVbo = read_temp( tt_p2 + 3 );
  Temp_PANsl = read_temp( tt_p2 + 5 );
  Temp_SF6sl = read_temp( tt_p2 + 1 );
  Temp_F11sl = read_temp( tt_p2 + 0 );
  Trap1_re  = read_ad_value( tt_p3 + 21, 10, 10, 0 );
  home->U1.Trap1_re = Trap1_re;
  home->U13.Temp_stack = Temp_stack;
  home->U15.Temp_SF6sl = Temp_SF6sl;
  home->U19.Temp_PANsrc = Temp_PANsrc;
  home->U17.Temp_PANsl = Temp_PANsl;
  home->U17.Temp_PANlmp = Temp_PANlmp;
  home->U17.Temp_Ipump = Temp_Ipump;
  home->U17.Temp_GSVbo = Temp_GSVbo;
  home->U13.Temp_GasBot = Temp_GasBot;
  home->U9.Temp_F11sl = Temp_F11sl;
  EF2_1();
}

static void BF32_23(void) {
  CF2_1();
  CurrFile_MSD = CurrFileMSD_;
  CurrInj_MSD = CurrInjMSD_;
  CO_ECD_Sp = OmegaArray( 2 * 0x6 - 1 );
  F11_ECD_Sp = OmegaArray( 2 * 0x5 - 1 );
  CO_Col_Sp = OmegaArray( 2 * 0x2 - 1 );
  F11_Col_Sp = OmegaArray( 2 * 0x1 - 1 );
  CO_ECD = OmegaArray( 2 * 0x6 - 2 );
  F11_ECD = OmegaArray( 2 * 0x5 - 2 );
  CO_Col = OmegaArray( 2 * 0x2 - 2 );
  F11_Col = OmegaArray( 2 * 0x1 - 2 );
  Trap2_re  = read_ad_value( tt_p3 + 23, 10, 10, 0 );
  Trap2_set = read_ad_value( tt_p3 + 22, 10, 10, 0 );
  Trap1_set = read_ad_value( tt_p3 + 20, 10, 10, 0 );
  CurrFile = CurrFile_;
  CurrInj = CurrInj_;
  home->U19.F11_ECD_Sp = F11_ECD_Sp;
  home->U19.F11_ECD = F11_ECD;
  home->U21.F11_Col_Sp = F11_Col_Sp;
  home->U21.F11_Col = F11_Col;
  home->U19.CurrInj_MSD = CurrInj_MSD;
  home->U0.CurrInj = CurrInj;
  home->U21.CurrFile_MSD = CurrFile_MSD;
  home->U0.CurrFile = CurrFile;
  home->U23.CO_ECD_Sp = CO_ECD_Sp;
  home->U23.CO_ECD = CO_ECD;
  home->U21.CO_Col_Sp = CO_Col_Sp;
  home->U21.CO_Col = CO_Col;
  home->U7.VoltBP_PAN = VoltBP_PAN;
  home->U9.Trap2_set = Trap2_set;
  home->U9.Trap2_re = Trap2_re;
  home->U5.Trap1_set = Trap1_set;
  EF2_1();
  { ITXFileNum = CurrInj * 1000 + CurrFile;
  }
}

static void BF32_25(void) {
  CF8_1();
  IonWinNum = cache_read(0x1000);
  PAN_ECD_Sp = OmegaArray( 2 * 0xA - 1 );
  SF6_ECD_Sp = OmegaArray( 2 * 0x7 - 1 );
  SF6_Col_Sp = OmegaArray( 2 * 0x3 - 1 );
  PAN_ECD = OmegaArray( 2 * 0xA - 2 );
  SF6_ECD = OmegaArray( 2 * 0x7 - 2 );
  SF6_Col = OmegaArray( 2 * 0x3 - 2 );
  PANsol4 = read_dig_out(7);
  PANsol3 = read_dig_out(6);
  PANsol2 = read_dig_out(5);
  PANsol1 = read_dig_out(4);
  PANflowCont = read_dig_out(19);
  PANlamp = read_dig_out(18);
  pelt2State = read_dig_out(17);
  pelt1State = read_dig_out(16);
  pilotTrip = read_dig_in(24);
  PANflsh = read_dig_out(1);
  InjState = GCstate;
  home->U21.SF6_ECD_Sp = SF6_ECD_Sp;
  home->U21.SF6_ECD = SF6_ECD;
  home->U23.SF6_Col_Sp = SF6_Col_Sp;
  home->U23.SF6_Col = SF6_Col;
  home->U0.pilotTrip = pilotTrip;
  home->U19.pelt2State = pelt2State;
  home->U0.pelt1State = pelt1State;
  home->U21.PANsol4 = PANsol4;
  home->U21.PANsol3 = PANsol3;
  home->U21.PANsol2 = PANsol2;
  home->U19.PANsol1 = PANsol1;
  home->U19.PANlamp = PANlamp;
  home->U19.PANflsh = PANflsh;
  home->U19.PANflowCont = PANflowCont;
  home->U21.PAN_ECD_Sp = PAN_ECD_Sp;
  home->U21.PAN_ECD = PAN_ECD;
  home->U19.IonWinNum = IonWinNum;
  home->U0.InjState = InjState;
  EF2_1();
}

static void BF32_27(void) {
  CF2_1();
  Trap2_CB_Sp = OmegaArray( 2 * 0x9 - 1 );
  Trap1_CB_Sp = OmegaArray( 2 * 0x8 - 1 );
  SF6_Post_Sp = OmegaArray( 2 * 0x4 - 1 );
  Trap2_CB = OmegaArray( 2 * 0x9 - 2 );
  Trap1_CB = OmegaArray( 2 * 0x8 - 2 );
  SF6_Post = OmegaArray( 2 * 0x4 - 2 );
  trap2relay = read_dig_out(23);
  trap1relay = read_dig_out(22);
  SLoopMSout = read_dig_out(3);
  SLoopOUT = read_dig_out(2);
  SLoopIN = read_dig_out(0);
  MSDstate = *MSDstatePtr;
  GCstate = *GCstatePtr;
  home->U19.MSDstate = MSDstate;
  home->U19.GCstate = GCstate;
  home->U19.trap2relay = trap2relay;
  home->U23.Trap2_CB_Sp = Trap2_CB_Sp;
  home->U23.Trap2_CB = Trap2_CB;
  home->U21.trap1relay = trap1relay;
  home->U21.Trap1_CB_Sp = Trap1_CB_Sp;
  home->U19.Trap1_CB = Trap1_CB;
  home->U19.SLoopOUT = SLoopOUT;
  home->U0.SLoopMSout = SLoopMSout;
  home->U19.SLoopIN = SLoopIN;
  home->U15.SF6_Post_Sp = SF6_Post_Sp;
  home->U15.SF6_Post = SF6_Post;
  EF2_1();
  if (!(V0)) {
    {
      if ( GCstate == 2 ) VSET_0_(3);
    }
  }
    else {
      if ( GCstate == 1 ) VSET_0_(2);
      if ( GCstate == 0 ) VSET_0_(1);
    }
  {
  		MSDstateS = MSDstate;
  	}
  if (!(V5)) {
    {
      if ( MSDstate == 2 ) VSET_2_(3);
    }
  }
    else {
      if ( MSDstate == 1 ) VSET_2_(2);
      else if ( MSDstate == 0 ) VSET_2_(1);
    }
}

static void BF64_29(void) {
  CF2_1();
  {
  
  	msdECUR     = MSDeng[msg_MONecur];
  	msdIPOL     = MSDeng[msg_MONipol];
  	msdREP      = MSDeng[msg_MONrep];
  	msdEEGY     = MSDeng[msg_MONeegy];
  	msdAMUG     = MSDeng[msg_MONamug];
  	msdAMUO     = MSDeng[msg_MONamuo];
  	msdIFOC     = MSDeng[msg_MONifoc];
  	msdENT      = MSDeng[msg_MONent];
  	msdFSEL     = MSDeng[msg_MONfsel];
  	msdMMW      = MSDeng[msg_MONmmw];
  	msdENTO     = MSDeng[msg_MONento];
  	msdHEDEN    = MSDeng[msg_MONheden];
  	msdCAL      = MSDeng[msg_MONcal];
  	msdEMUL     = MSDeng[msg_MONemul];
  	msdSRCTEMP  = MSDeng[msg_MONsrctemp];
  	msdQUAD     = MSDeng[msg_MONquad];
  	msdTURBOPCT = MSDeng[msg_MONturbopct];
  	msdMASG     = MSDeng[msg_MONmasg];
  	msdMASO     = MSDeng[msg_MONmaso];
  
  	if ( MSDeng_proxy != 0 )
  		Trigger( MSDeng_proxy );
  
  }
  home->U31.msdSRCTEMP = msdSRCTEMP;
  home->U31.msdQUAD = msdQUAD;
  home->U33.msdTURBOPCT = msdTURBOPCT;
  home->U33.msdMASG = msdMASG;
  home->U27.msdMASO = msdMASO;
  EF2_1();
}

static void BF64_31(void) {
  CF2_1();
  home->U29.msdMMW = msdMMW;
  home->U29.msdENTO = msdENTO;
  home->U31.msdHEDEN = msdHEDEN;
  home->U31.msdCAL = msdCAL;
  home->U25.msdEMUL = msdEMUL;
  EF2_1();
}

static void BF64_61(void) {
  CF2_1();
  home->U27.msdAMUG = msdAMUG;
  home->U27.msdAMUO = msdAMUO;
  home->U29.msdIFOC = msdIFOC;
  home->U29.msdENT = msdENT;
  home->U23.msdFSEL = msdFSEL;
  EF2_1();
}

static void BF64_63(void) {
  CF2_1();
  home->U25.msdECUR = msdECUR;
  home->U27.msdIPOL = msdIPOL;
  home->U27.msdREP = msdREP;
  home->U21.msdEEGY = msdEEGY;
  EF2_1();
}

static void (*efuncs[64])() = {
  BF32_0,
  BF32_1,
  BF2_0,
  BF32_3,
  BF2_0,
  BF32_5,
  BF2_0,
  BF32_7,
  BF2_0,
  BF32_9,
  BF2_0,
  BF32_11,
  BF2_0,
  BF32_13,
  BF2_0,
  BF32_15,
  BF2_0,
  BF32_17,
  BF2_0,
  BF32_19,
  BF2_0,
  BF32_21,
  BF2_0,
  BF32_23,
  BF2_0,
  BF32_25,
  BF2_0,
  BF32_27,
  BF2_0,
  BF64_29,
  BF2_0,
  BF64_31,
  BF32_0,
  BF32_1,
  BF2_0,
  BF32_3,
  BF2_0,
  BF32_5,
  BF2_0,
  BF32_7,
  BF2_0,
  BF32_9,
  BF2_0,
  BF32_11,
  BF2_0,
  BF32_13,
  BF2_0,
  BF32_15,
  BF2_0,
  BF32_17,
  BF2_0,
  BF32_19,
  BF2_0,
  BF32_21,
  BF2_0,
  BF32_23,
  BF2_0,
  BF32_25,
  BF2_0,
  BF32_27,
  BF2_0,
  BF64_61,
  BF2_0,
  BF64_63
};

#define TRN 320
#define TRD 1
#define LOWLIM (-2880)
#define HIGHLIM (2880)
#define NBROW 48
#define NROWMINF 1
#define NSECSPER 1
#define NROWSPER 32
#define SYNCHVAL 0xABB4
#define NROWMAJF 64
#define MFSECNUM 32
#define MFSECDEN 1
#define SECDRIFT 90

/* Some temporary defs until everything is in the right place */
#ifndef TS_MFC_LIMIT
  #define TS_MFC_LIMIT 32767
#endif

/* This is for real: */
#ifndef DG_OTHER_CASES
  #define DG_OTHER_CASES
#endif

/* for debugging */
int check_ts = 1;

#if (NROWMAJF < 16)
  #define DP_NROWS 16
#else
  #define DP_NROWS NROWMAJF
#endif
#define NTS 3
#define incmod(x,y) if (x==((y)-1)) x = 0; else x++

static unsigned char dpdata[DP_NROWS][NBROW];
static tstamp_type tstamps[NTS], *tsps[DP_NROWS], *curts;
static unsigned short next_ts_index;
static unsigned short trans_row, col_row;
static short rowlets;
static short ts_checks;
#ifdef NEED_TIME_FUNCS
  static unsigned short CurMFC;
#endif
#define TSCHK_RTIME 1
#define TSCHK_IMPLICIT 2
#define TSCHK_CHECK 4
#define TSCHK_REQUIRED 8
static unsigned short next_minor_frame, majf_row;
#if (NROWMINF != 1)
  static unsigned short minf_row;
#endif
#ifndef DG_CUSTOM_INIT
  #define DG_CUSTOM_INIT
#endif
#ifndef TMINITFUNC
  #define TMINITFUNC
#endif
#ifndef ONINT_INIT
  #define ONINT_INIT
#endif
#ifndef ONINT_TERM
  #define ONINT_TERM
#endif
#ifndef DG_CUSTOM_TERM
  #define DG_CUSTOM_TERM
#endif


void main(int argc, char **argv) {
  oui_init_options(argc, argv);
  { int nlrsp;
	
	nlrsp = set_response(NLRSP_QUIET);
	if (find_CC(0) != -1)
	  cc_init_options(argc, argv, DCT_TM, DCT_TM, 0, 0, FORWARD_QUIT);
	set_response(nlrsp);
  }
  DG_CUSTOM_INIT;
  TMINITFUNC;
  BEGIN_MSG;
  DG_operate();
  DG_CUSTOM_TERM;
  nl_error(MSG_EXIT_NORM, "Terminated");
}


static void ts_check(void) {
  rowlets -= TRN;
  if (rowlets < LOWLIM || rowlets > HIGHLIM)
	ts_checks = TSCHK_RTIME | TSCHK_CHECK;
}

static void Collect_row(void) {
  time_t rtime, dt;
  unsigned int dmfc;
  tstamp_type *newts;
  
  #ifdef _SUBBUS_H
	tick_sic();
  #endif
  #if (NROWMINF != 1)
	if (minf_row == 0) {
  #endif
	  if ((ts_checks & TSCHK_REQUIRED) == 0 && (
	  #ifdef ROLLOVER
		  next_minor_frame == ROLLOVER ||
	  #endif
		  next_minor_frame - curts->mfc_num > TS_MFC_LIMIT))
		ts_checks |= TSCHK_IMPLICIT | TSCHK_REQUIRED;
	  if (ts_checks) {
		newts = &tstamps[next_ts_index];
		if (newts == curts)
		  nl_error(MSG_EXIT_ABNORM, "TimeStamp overflow");
		if (ts_checks & TSCHK_RTIME)
		  rtime = time(NULL);
		if (ts_checks & (TSCHK_IMPLICIT|TSCHK_CHECK)) {
		  /* mfc rate is MFSECNUM/MFSECDEN mfcs/second, so
		     mfcs/MFSECNUM = secs/MFSECDEN = dmfc
		  */
		  #if (MFSECDEN == 1)
			#define MFSECDENMUL
		  #else
			#define MFSECDENMUL * MFSECDEN
		  #endif
		  dmfc = next_minor_frame - curts->mfc_num;
		  #if (MFSECNUM != 1)
			dmfc /= MFSECNUM;
		  #endif
		  newts->secs = curts->secs + dmfc MFSECDENMUL;
		  #if (MFSECNUM != 1)
			newts->mfc_num = curts->mfc_num + dmfc*MFSECNUM;
		  #else
			newts->mfc_num = next_minor_frame;
		  #endif
		  if (ts_checks & TSCHK_CHECK) {
			/* compare rtime to newts->secs */
			#if (MFSECNUM != 1)
			  dmfc = next_minor_frame - newts->mfc_num;
			  dt = dmfc MFSECDENMUL / MFSECNUM;
			  dt = rtime - dt - newts->secs;
			#else
			  dt = rtime - newts->secs;
			#endif
			if (dt > SECDRIFT || dt < -SECDRIFT)
			  ts_checks |= TSCHK_REQUIRED;
		  }
		}
		if (ts_checks & TSCHK_REQUIRED) {
		  if (ts_checks & TSCHK_RTIME) {
			/* New time stamp:
			  If higher resolution time is available {
				f = fraction of a second extra
				rmfc = rate of mfc Hz
				f*rmfc = number of minor frames since even second.
				record floor(seconds) and mfc - f*rmfc
			  }
			*/
			newts->secs = rtime;
			newts->mfc_num = next_minor_frame;
		  }
		  #ifdef ROLLOVER
			if (next_minor_frame == ROLLOVER) {
			  next_minor_frame = 0;
			  newts->mfc_num -= ROLLOVER;
			}
		  #endif
		  incmod(next_ts_index, NTS);
		  tsps[col_row] = curts = newts;
		} else tsps[col_row] = NULL;
		ts_checks = 0;
	  } else tsps[col_row] = NULL;
	  MFCtr = next_minor_frame;
	  #ifdef NEED_TIME_FUNCS
		CurMFC = MFCtr;
	  #endif
	  next_minor_frame++;
  #if (NROWMINF != 1)
	  minf_row = 1;
    } else
	  #if (NROWMINF > 2)
		if (minf_row == NROWMINF-1)
	  #endif
  #endif
	{ /* Synch Calculations */
      #ifdef INVSYNCH
		if (majf_row == NROWMAJF-1)
		  Synch = ~SYNCHVAL;
		else
	  #else
		  Synch = SYNCHVAL;
	  #endif
  #if (NROWMINF != 1)
	  minf_row = 0;
  #endif
	}
  #if (NROWMINF > 2)
	  else ++minf_row;
  #endif
  
  /* appropriate collection function */
  home = (void *) dpdata[col_row];
  efuncs[majf_row]();
  incmod(majf_row, NROWMAJF);
  rowlets += TRD;
  
  incmod(col_row, DP_NROWS);
  if (col_row == trans_row)
	nl_error(MSG_EXIT_ABNORM, "Collection overflow");
}

/* time_prox holds information pertinent to the two timing proxies.
   If proxy == 0, the proxy has not been initialized.
   If proxy != 0, timer_type indicates what type of timer is used.
   TPTYPE_SYS means u.timer is a system timer.
   TPTYPE_TBD means u.tmrno is a timer board timer.
   TPTYPE_EXT means an external timing source is used.
*/
struct time_prox {
  pid_t proxy;
  int timer_type;
  union {
	timer_t timer;
	int tmrno;
  } u;
};
#define TPTYPE_SYS 0
#define TPTYPE_TBD 1
#define TPTYPE_EXT 2
static struct time_prox col_timer = {0,0}, ts_timer = {0,0};

#define NSEC (1000000000)

static void start_timing(int rn, int rd, struct time_prox *tp,
						 unsigned char msgtxt) {
  struct itimercb tcb;
  struct itimerspec tval, otval;

  assert(tp->proxy == 0);
  if (rn <= 4*rd) {
	set_response(0);
	tp->u.tmrno = Tmr_proxy(3, rn*(TMR_0_FREQ/rd), msgtxt);
	set_response(3);
	if (tp->u.tmrno == -1) nl_error(0, "Using System Timer");
	else {
	  tp->proxy = -1;
	  tp->timer_type = TPTYPE_TBD;
	  nl_error(0, "Using Timer Board Timer");
	  return;
	}
  }
  tp->proxy = nl_make_proxy(&msgtxt, 1);
  tcb.itcb_event.evt_value = tp->proxy;
  tp->u.timer = mktimer(TIMEOFDAY, _TNOTIFY_PROXY, &tcb);
  if (tp->u.timer == -1)
	nl_error(MSG_EXIT_ABNORM, "Error making timer: %d", tp->u.timer);
  tval.it_value.tv_sec = rn/rd;
  tval.it_value.tv_nsec = (rn%rd) * (NSEC/rd);
  tval.it_interval = tval.it_value;
  if (reltimer(tp->u.timer, &tval, &otval) == -1)
	nl_error(MSG_EXIT_ABNORM, "Error in reltimer: %d\n", errno);
  tp->timer_type = TPTYPE_SYS;
}

/* Only detaches proxy if timing is internal */
static void stop_timing(struct time_prox *tp) {
  if (tp->proxy != 0) {
	switch (tp->timer_type) {
	  case TPTYPE_TBD:
		Tmr_reset(tp->u.tmrno);
		tp->proxy = 0;
		break;
	  case TPTYPE_SYS:
		rmtimer(tp->u.timer);
		tp->u.timer = 0;
		if (tp->proxy != 0) {
		  qnx_proxy_detach(tp->proxy);
		  tp->proxy = 0;
		}
		break;
	  default:
		break;
	}
  }
}

#define MKDCTV(x,y) ((x<<8)|y)
#define DCTV(x,y) MKDCTV(DCT_##x,DCV_##y)

int DG_DASCmd(unsigned char type, unsigned char val) {
  switch (MKDCTV(type, val)) {
	case DCTV(TM,TM_START):
	  if (check_ts)
		start_timing(10, 1, &ts_timer, COL_TSPROXY);
	  if (col_timer.proxy == 0)
		start_timing(tmi(nsecsper), tmi(nrowsper), &col_timer, COL_REG_SIG);
	  ts_checks = TSCHK_RTIME | TSCHK_REQUIRED;
	  next_ts_index = 0;
	  curts = NULL;
	  trans_row = col_row = 0;
	  rowlets = 0;
	  next_minor_frame = majf_row = 0;
	  #if (NROWMINF != 1)
		minf_row = 0;
	  #endif
	  ONINT_INIT
	  break;
	case DCTV(TM,TM_END):
	case DCTV(QUIT,QUIT):
	  #ifdef _SUBBUS_H
		disarm_sic();
	  #endif
	  ONINT_TERM
	  stop_timing(&col_timer);
	  stop_timing(&ts_timer);
	  break;
	default:
	  break;
  }
  return(0);
}


#include <sys/seginfo.h>

/* I've added some verbosity here, but I'd like to add more:
   The messages should include the pointer ID, which we don't know here.
   I'd have to change tmc together with this file...
*/
static void COL_get_pointer(pid_t sent_tid, void far **pptr, void far *ptr) {
  unsigned segt;
  
  if (*pptr == &nullregion) {
	segt = qnx_segment_get(sent_tid, FP_SEG(ptr), 0);
	if (segt != -1) {
	  *pptr = MK_FP(segt, FP_OFF(ptr));
	  reply_byte(sent_tid, DAS_OK);
	  nl_error(0, "Pointer set from pid %d", sent_tid);
	  return;
	} else nl_error(MSG_WARN, "Pointer failed from pid %d", sent_tid);
  } else nl_error(MSG_WARN, "Pointer from pid %d previously set", sent_tid);
  reply_byte(sent_tid, DAS_UNKN);
}

static void COL_free_pointer(pid_t sent_tid, void far **pptr) {
  if (*pptr != &nullregion) {
	qnx_segment_free( FP_SEG(*pptr));
	*pptr = &nullregion;
	reply_byte(sent_tid, DAS_OK);
  } else reply_byte(sent_tid, DAS_UNKN);
}

static void COL_recv_proxy(pid_t sent_tid, pid_t *prx, pid_t proxy) {
  if (*prx == 0) {
	*prx = proxy;
	reply_byte(sent_tid, DAS_OK);
  } else reply_byte(sent_tid, DAS_UNKN);
}

static void COL_end_proxy(pid_t sent_tid, pid_t *prx, struct colmsg *c) {
  c->u.proxy = *prx;
  c->type = *prx == 0 ? DAS_UNKN : DAS_OK;
  *prx = 0;
  Reply(sent_tid, c, sizeof(struct colmsg));
}

#include <stddef.h>

static void read_col_send( void *dest, size_t length,
					struct colmsg *cmsg, pid_t sent_tid ) {
  size_t trulen = min( length, cmsg->u.data.size );
  if ( trulen <= MAX_COLMSG_SIZE-5 ) {
	memcpy( dest, cmsg->u.data.data, trulen );
  } else {
	Readmsg( sent_tid,
	  offsetof( struct colmsg, u.data.data ), dest, trulen );
  }
}

int DG_other(unsigned char *msg_ptr, int sent_tid) {
  struct colmsg *cmsg;
  
  switch (*msg_ptr) {
	case COL_REG_SIG:
	  if (dbr_info.tm_started) Collect_row();
	  break;
	case COL_TSPROXY:
	  if (dbr_info.tm_started) ts_check();
	  break;
	case COL_REGULATE:
	  {
		struct dgreg_rep dgregrep;
		unsigned char msgtxt = COL_REG_SIG;

		if (col_timer.proxy != 0) {
		  dgregrep.reply_code = DAS_BUSY;
		  dgregrep.proxy = col_timer.proxy;
		} else {
		  col_timer.proxy = nl_make_proxy(&msgtxt, 1);
		  col_timer.timer_type = TPTYPE_EXT;
		  dgregrep.proxy = col_timer.proxy;
		  dgregrep.reply_code = DAS_OK;
		}
		Reply(sent_tid, &dgregrep, sizeof(dgregrep));
	  }
	  return(0);
	case COL_SET_POINTER:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 5: COL_get_pointer(sent_tid, &MSDstatePtr, cmsg->u.pointer); break;
		case 4: COL_get_pointer(sent_tid, &GCstatePtr, cmsg->u.pointer); break;
		default: return(reply_byte(sent_tid, DAS_UNKN));
	  }
	  return(0);
	case COL_RESET_POINTER:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 5: COL_free_pointer(sent_tid, &MSDstatePtr); break;
		case 4: COL_free_pointer(sent_tid, &GCstatePtr); break;
		default: return(reply_byte(sent_tid, DAS_UNKN));
	  }
	  return(0);
	case COL_SET_PROXY:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 12: COL_recv_proxy(sent_tid, &maycomm_proxy, cmsg->u.proxy); break;
		case 7: COL_recv_proxy(sent_tid, &MSDeng_proxy, cmsg->u.proxy); break;
		case 6: COL_recv_proxy(sent_tid, &MSDresp_proxy, cmsg->u.proxy); break;
		case 4: COL_recv_proxy(sent_tid, &valco_proxy, cmsg->u.proxy); break;
		case 8: COL_recv_proxy(sent_tid, &ECDresp_proxy, cmsg->u.proxy); break;
		case 3: COL_recv_proxy(sent_tid, &rs485_proxy, cmsg->u.proxy); break;
		default: return(reply_byte(sent_tid,DAS_UNKN));
	  }
	  return(0);
	case COL_RESET_PROXY:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case 12: COL_end_proxy(sent_tid, &maycomm_proxy, cmsg); break;
		case 7: COL_end_proxy(sent_tid, &MSDeng_proxy, cmsg); break;
		case 6: COL_end_proxy(sent_tid, &MSDresp_proxy, cmsg); break;
		case 4: COL_end_proxy(sent_tid, &valco_proxy, cmsg); break;
		case 8: COL_end_proxy(sent_tid, &ECDresp_proxy, cmsg); break;
		case 3: COL_end_proxy(sent_tid, &rs485_proxy, cmsg); break;
		default: return(reply_byte(sent_tid,DAS_UNKN));
	  }
	  return(0);
	case COL_SEND:
	  cmsg = (struct colmsg *)msg_ptr;
	  switch (cmsg->id) {
		case COL_SEND_INIT:
		  if (stricmp(cmsg->u.name, "tdl") == 0) {
			cmsg->u.data.id = 6;
			cmsg->u.data.size = sizeof(tdl);
		  } else if (stricmp(cmsg->u.name, "MSDeng") == 0) {
			cmsg->u.data.id = 5;
			cmsg->u.data.size = sizeof(MSDeng);
		  } else if (stricmp(cmsg->u.name, "MSDresp") == 0) {
			cmsg->u.data.id = 4;
			cmsg->u.data.size = sizeof(MSDresp);
		  } else if (stricmp(cmsg->u.name, "valvePos") == 0) {
			cmsg->u.data.id = 3;
			cmsg->u.data.size = sizeof(valvePos);
		  } else if (stricmp(cmsg->u.name, "ECDresp") == 0) {
			cmsg->u.data.id = 2;
			cmsg->u.data.size = sizeof(ECDresp);
		  } else if (stricmp(cmsg->u.name, "Omega485") == 0) {
			cmsg->u.data.id = 1;
			cmsg->u.data.size = sizeof(Omega485);
		  } else return reply_byte(sent_tid,DAS_UNKN);
		  cmsg->type = DAS_OK;
		  Reply(sent_tid, cmsg, offsetof(struct colmsg, u.data.data));
		  return 0;
		case COL_SEND_SEND:
		  switch (cmsg->u.data.id) {
			case 6:
			  read_col_send(&tdl, sizeof(tdl), cmsg, sent_tid );
			  break;
			case 5:
			  read_col_send(&MSDeng, sizeof(MSDeng), cmsg, sent_tid );
			  break;
			case 4:
			  read_col_send(&MSDresp, sizeof(MSDresp), cmsg, sent_tid );
			  break;
			case 3:
			  read_col_send(&valvePos, sizeof(valvePos), cmsg, sent_tid );
			  break;
			case 2:
			  read_col_send(&ECDresp, sizeof(ECDresp), cmsg, sent_tid );
			  break;
			case 1:
			  read_col_send(&Omega485, sizeof(Omega485), cmsg, sent_tid );
			  break;
			default: return reply_byte(sent_tid, DAS_UNKN);
		  }
		  break;
		case COL_SEND_RESET: break;
		default: return reply_byte(sent_tid,DAS_UNKN);
	  }
	  return reply_byte(sent_tid, DAS_OK);
	DG_OTHER_CASES /* defined via preprocessor elsewhere */
	default:
	  return(reply_byte(sent_tid, DAS_UNKN));
  }
  /* We only get here on the two proxy calls: proxies don't need reply */
  return(0);
}

/* Need a 'current time stamp' different from the one in dbr_info,
   since that one pertains to stuff already being transmitted.
   Need to optimize for wrap-around when DG_s_data is augmented
   to support two blocks of data.
*/
int DG_get_data(unsigned int n_rows) {
  int rowlim, i;
  
  if (trans_row != col_row) {
	if (tsps[trans_row] != NULL) {
	  DG_s_tstamp(tsps[trans_row]);
	  tsps[trans_row] = NULL;
	} else {
	  rowlim = trans_row + n_rows;
	  if (DP_NROWS < rowlim) rowlim = DP_NROWS;
	  if (col_row > trans_row && col_row < rowlim) rowlim = col_row;
	  for (i = trans_row; i < rowlim && tsps[i] == NULL; i++);
	  assert(i != trans_row);
	  DG_s_data(i-trans_row, dpdata[trans_row], 0, NULL);
	  trans_row = i;
	  if (trans_row == DP_NROWS) trans_row = 0;
	}
  }
  return(0);
}

#ifdef NEED_TIME_FUNCS
  #if (NROWMINF != 1)
	#define ROWS(x) (((unsigned long)(x))*NROWMINF+minf_row)
  #else
	#define ROWS(x) (x)
  #endif
  #if (NSECSPER != 1)
	#define FRACSECS(x) (((unsigned long)ROWS(x))*NSECSPER)
  #else
	#define FRACSECS(x) ROWS(x)
  #endif
  long itime(void) {
	  return(dbr_info.t_stmp.secs +
		FRACSECS(CurMFC-dbr_info.t_stmp.mfc_num)
	#if (NROWSPER != 1)
		  / NROWSPER
	#endif
		  );
  }
  double dtime(void) {
	  return(dbr_info.t_stmp.secs +
		(double) FRACSECS(CurMFC-dbr_info.t_stmp.mfc_num)
	#if (NROWSPER != 1)
		  / NROWSPER
	#endif
		  );
  }
  double etime(void) {
	double t;
	static double t0 = -1e9;
	
	t = dtime();
	if (t0 == -1e9) t0 = t;
	return(t - t0);
  }
#endif

dbr_info_type dbr_info = {
  "", /* tmid */
  48, /* NBMINF */
  48, /* NBROW */
  64, /* NROWSMAJF */
  1, /* NSECSPER */
  32, /* NROWSPER */
  0, 1, /* MFC lsb col, msb col */
  0xABB4, /* Synch Value */
  0 /* not inverted */
};
